#include "linserver.h"
#include "token.h"
#include <stdlib.h>
#include <string.h>

// In all replies 
// 	If reply begins with '-' means error occured while processing
//	request, error details are provided after '-'.
//	Reply beginning with '+' imply success and provide more descriptive
//	message after '+'. The last line of '+'s has '*' as first character
bool serviceRequest(QSocket &client,char *command) {

	char param[2][1024];
	int totalTokens;

	// *** LISTALL ***	
	if (!strcmp(command,"rollback")) {
		if (config.rollback()) client.write("*Done\r\n");
		else client.write("-Error: No entries\r\n");
	}
	
	if (!strcmp(command,"next")) {
		if (!config.next()) {
			client.write("-Error: No more entries\r\n");
			return true;
		}
		sprintf(command,"+%d:%s:%s:%d:%s:%s\r\n",
		config.current->port,
		config.current->protocol==LIN_TCP ? "tcp" : "udp",
		config.current->serverType==LIN_WAIT ? "wait" : "nowait",
		config.current->backlog,config.current->accessFile,
		config.current->serviceFile);
		client.write(command);
	}

	if (!strcmp(command,"listall")) {
		if (config.rollback())
			do {
				sprintf(command,"+%d:%s:%s:%d:%s:%s\r\n",
					config.current->port,
					config.current->protocol==LIN_TCP ? "tcp" : "udp",
					config.current->serverType==LIN_WAIT ? "wait" : "nowait",
					config.current->backlog,
					config.current->accessFile,
					config.current->serviceFile);
				client.write(command);
			} while(config.next());
		client.write("*Done\r\n");
	}
	else if (!strcmp(command,"exit")) return false;

	QToken token;
	token.set(command," "); token.setMultiDelim(true);
	totalTokens=token.total();
	if (!token.get(param[0])) return true;

	// *** LISTACCESS PORT TCP|UDP ***
	if (!strcmp(param[0],"listaccess")) {
		if (totalTokens!=3) {
			client.write("-Error: listaccess port tcp|udp\r\n");
			return true;
		}
		token.get(param[0]);
		token.get(param[1]);
		if (strcmp(param[1],"tcp") && strcmp(param[1],"udp")) {
			client.write("-Error: tcp|udp expected\r\n");
			return true;
		}
		if (!config.get(atoi(param[0]),strcmp(param[1],"tcp") ? LIN_UDP : LIN_TCP)) {
				client.write("-Error: Unable to locate service\r\n");
			return true;
		}
		if (config.rollbackAccess()) {
			do {
				sprintf(command,"+%s:%s\r\n",
				config.currentAccess->access==LIN_ALLOW ? "allow" : "deny",
				config.currentAccess->ipFile);
				client.write(command);
			} while(config.nextAccess());
		}
		client.write("*Done\r\n");
	}
	
	// *** LISTSERVICE PORT TCP|UDP ***
	if (!strcmp(param[0],"listservice")) {
		if (totalTokens!=3) {
			client.write("-Error: listservice port tcp|udp\r\n");
			return true;
		}
		token.get(param[0]); token.get(param[1]);
		if (!config.get(atoi(param[0]),strcmp(param[1],"tcp")?LIN_UDP:LIN_TCP)) {
			client.write("-Error: Unable to locate service\r\n");
			return true;
		}
		
		if (config.rollbackService()) {
			do {
				sprintf(command,"+%s:%d:%s:%s:%s:%s:%s\r\n",
					config.currentService->ipFile,
					config.currentService->uid,
					config.currentService->service,
					config.currentService->fields[0],
					config.currentService->fields[1],
					config.currentService->fields[2],
					config.currentService->fields[3]);
				client.write(command);
			} while(config.nextService());
		}
		client.write("*Done\r\n");
	}

	// *** LISTGROUP ***
	if (!strcmp(param[0],"listgroup")) {
		if (totalTokens!=2) {
			client.write("-Error: listgroup group-name\r\n");
			return true;
		}
		token.get(param[0]);
		FILE *file=fopen(param[0],"r");
		if (!file) {
			client.write("-Error: Invalid group name\r\n");
			return true;
		}
		while(!feof(file)) {
			param[1][0]='+';
			fgets(&param[1][1],1024,file);
			client.write(param[1]);
		}
		fclose(file);
		client.write("*Done\r\n");
	}


	return true;	
}

		
void linAdmin(int handle) {
	QSocket client;
	client.attach(handle);
	client.setException(true);

	char command[1024];
	int totalTokens;
	
	try {
		client.write("*");
		client.write(VERSION);
		client.write("\r\n");
		while(1) {
			client.readline(command,1024);
			if (!serviceRequest(client,command))
				return;
		}
	}
	catch(...) {
		return;
	}
	return;
}

		
